@@ -29,7 +29,7 @@ class Role8 extends Role7 {
   }
 
   public function getPerms() {
-    $role = entity_load('user_role', $this->rid);
+    $role = Role::load($this->rid);
     $perms = $role->getPermissions();
     // $perms = user_role_permissions(array($this->rid => $this->name));
     return $perms;
@@ -52,7 +52,7 @@ class Role8 extends Role7 {
   }
 
   public function delete() {
-    $role = entity_load('user_role', $this->rid);
+    $role = Role::load($this->rid);
     $role->delete();
   }
 
